package com.your_domain.your_product.employee.dao;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;

import com.jdbaccess.da.DAFactory;
import com.jdbaccess.da.Select;
import com.jdbaccess.da.Result;
import com.jdbaccess.da.Transaction;
import com.jdbaccess.da.Update;
import com.jdbaccess.dao.DataAccessObjectImpl;
import com.jdbaccess.exception.ApplicationException;
import com.jdbaccess.sql.ComparisonCondition;
import com.jdbaccess.sql.LogicalCondition;
import com.jdbaccess.transfer.Row;
import com.jdbaccess.transfer.TransferObject;
import com.your_domain.your_product.address.transfer.Address;
import com.your_domain.your_product.employee.transfer.EmpAddr;
import com.your_domain.your_product.employee.transfer.Employee;

/**
 * persistence layer: dao class of entity employee 
 */
public class EmployeeDaoImpl extends DataAccessObjectImpl implements EmployeeDao {
  private static String ENTITY = Employee.getEntityName();
  private static String ENTITY_ADDRESS = Address.getEntityName();
  private static String ENTITY_EMP_ADDR = EmpAddr.getEntityName();
  private static String SEQUENCE = Employee.getSeqName();
  private static String CREATE_TABLE_STRING = 
    "create table " + ENTITY + 
    " (id number(15,0) primary key, name varchar2(300) unique, salary number, description clob, picture blob, creation date, modification date)";

  public EmployeeDaoImpl() {
    super(); // set the appropriate DAReader
  }

  public EmployeeDaoImpl(Transaction t) {
    super(t); // set the appropriate DAReader and transaction
  }

  public void dropTable() throws ApplicationException {
    dropTable(ENTITY);  
  }
  
  public void dropSequence() throws ApplicationException {
    dropSequence(SEQUENCE);  
  }
  
  public void createSequence() throws ApplicationException {
    createSequence(SEQUENCE);  
  }
  
  public void createTable() throws ApplicationException {
    createTable(CREATE_TABLE_STRING);  
  }
  
  public void createPackage() throws ApplicationException {
    createPackageHead();
    createPackageBody();
  }
  
  public Employee insert(Employee emp) throws ApplicationException {
    ArrayList transferObjects = new ArrayList();
    transferObjects.add(emp);
    ArrayList retTransferObjects = (ArrayList)insert(transferObjects);
    return (Employee) retTransferObjects.get(0);
  }
  
  public ArrayList insert(ArrayList transferObjects) throws ApplicationException {
    for (int i=0; i<transferObjects.size(); i++) {
      TransferObject to = (TransferObject) transferObjects.get(i);
      to.setEntity(ENTITY);
      to.setSequenceName(SEQUENCE);
    }
    return super.insert(transferObjects);
  }

  public Employee insert(String name, BigDecimal salary, String description, byte[] picture, Date creation, Date modification) throws ApplicationException {
    Employee emp = new Employee();
    emp.setValueModified("name", name);
    emp.setValueModified("salary", salary);
    emp.setValueModified("description", description);
    emp.setValueModified("picture", picture);
    emp.setValueModified("creation", creation);
    emp.setValueModified("modification", modification);
    return insert(emp);
  }
  
  public Long updateSalaryById(Long id, Number salary) throws ApplicationException {
    Row row = new Row();
    row.setEntity(ENTITY);
    row.setValue("id", id);
    row.setValueModified("salary", salary);
    return update(row);
  }
  
  public Long deleteAll() throws ApplicationException {
    return super.deleteAll(ENTITY);
  }
  
  public Long updateSalaryByName(String name, Number salary) throws ApplicationException {
    Row row = new Row();
    row.setEntity(ENTITY);
    row.setValue("name", name);
    row.setValueModified("salary", salary);
    ComparisonCondition nameEqualsComparison = new ComparisonCondition("name", ComparisonCondition.EQUALS, null);
    LogicalCondition nameEqualsCondition = new LogicalCondition(nameEqualsComparison, LogicalCondition.EMPTY, null);
    row.setWhereCondition(nameEqualsCondition);
    return update(row);
  }
  
  public Result findAll(ArrayList orderBy) throws ApplicationException {
    return findAll(ENTITY, Employee.class, orderBy);
  }
  
  public Employee findById(Long id) throws ApplicationException {
    return (Employee) findByFieldName(ENTITY, "id", id, Employee.class);
  }
  
  public Result findByName(String name) throws ApplicationException {
    Select s = DAFactory.getSelect(transaction);
    String selectName = "byName";
    String select = "select * from " + ENTITY + " where name = ?";
    s.setObjectType(ENTITY);
    s.setName(selectName);
    s.setSql(select);
    s.setResultType(Employee.class);
    ArrayList params = new ArrayList();
    params.add(name);
    s.setParameters(params);
    Result r = s.execute();
    r.setReadLobsFull(true);
    return r;
  }
  
  public Result findBySalaryLessThan(Number salary) throws ApplicationException {
    Select s = DAFactory.getSelect(transaction);
    String selectName = "bySalary";
    String select = "select * from " + ENTITY + " where salary < ?";
    s.setObjectType(ENTITY);
    s.setName(selectName);
    s.setSql(select);
    s.setResultType(Employee.class);
    ArrayList params = new ArrayList();
    params.add(salary);
    s.setParameters(params);
    Result r = s.execute();
    r.setReadLobsFull(true);
    return r;
  }
  
  public ArrayList findAddressesByEmpId(Long empId) throws ApplicationException {
    Select s = DAFactory.getSelect(transaction);
    String selectName = "addressesByEmpId";
    String select = "select * from " + ENTITY_ADDRESS + 
                   " where id in (select addrId from " + ENTITY_EMP_ADDR + " where empId = ?)";
    s.setObjectType(ENTITY_ADDRESS);
    s.setName(selectName);
    s.setSql(select);
    s.setResultType(Address.class);
    ArrayList params = new ArrayList();
    params.add(empId);
    s.setParameters(params);
    Result r = s.execute();
    ArrayList addresses = r.getAllElements();
    r.close();
    return addresses;
  }
  
  private void createPackageHead() throws ApplicationException {
    Update u = DAFactory.getUpdate(transaction);
    u.setObjectType("employee");
    u.setName("createPackage");
    String createPackage = "CREATE OR REPLACE PACKAGE pkg_employee AS " +
                               "TYPE ref_cursor IS REF CURSOR; " +
                               "PROCEDURE prc_addSalaryPercent ( " + 
                                  "SALARY_PERCENT IN NUMBER," +
                                  "SUCCESSSTRING OUT VARCHAR2," +
                                  "ADDED_SALARY OUT NUMBER," +
                                  "EMPLOYEES_BEFORE OUT REF_CURSOR," +
                                  "EMPLOYEES_AFTER OUT REF_CURSOR" +
                               "); " +
                               "FUNCTION fnc_addSalaryPercent ( " + 
                                  "SALARY_PERCENT IN NUMBER" +
                               ") RETURN NUMBER; " +
                               "END pkg_employee; ";
    u.setSql(createPackage);
    u.execute();
    u.end();
  }

  private void createPackageBody() throws ApplicationException {
    Update u = DAFactory.getUpdate(transaction);
    u.setObjectType("employee");
    u.setName("createPackageBody");
    String createPackageBody = 
                "CREATE OR REPLACE " +
                "PACKAGE BODY pkg_employee AS " +
                  "PROCEDURE prc_addSalaryPercent ( " +
                    "SALARY_PERCENT IN NUMBER," +
                    "SUCCESSSTRING OUT VARCHAR2," +
                    "ADDED_SALARY OUT NUMBER," +
                    "EMPLOYEES_BEFORE OUT REF_CURSOR," +
                    "EMPLOYEES_AFTER OUT REF_CURSOR" +
                  ") IS " +
                    "rows_processed NUMBER;" +
                  "BEGIN " +
                    "OPEN EMPLOYEES_BEFORE FOR select * from employee;" +
                    "ADDED_SALARY := 0;" +
                    "rows_processed := 0; " +
                    "FOR emp IN (SELECT * FROM employee) LOOP " +
                      "rows_processed := rows_processed + 1;" +
                      "ADDED_SALARY := ADDED_SALARY + emp.salary / 100 * SALARY_PERCENT;" +
                    "END LOOP;" +
                    "UPDATE employee SET salary = salary + salary / 100 * SALARY_PERCENT;" +
                    "IF rows_processed > 0 THEN " +
                      "SUCCESSSTRING := 'Congratulations: Update of ' || rows_processed || ' employees by ' || SALARY_PERCENT || ' %';" + 
                    "END IF;" +
                    "OPEN EMPLOYEES_AFTER FOR select * from employee; " +
                  "END; " +
                  "FUNCTION fnc_addSalaryPercent ( " + 
                    "SALARY_PERCENT IN NUMBER " +
                  ") RETURN NUMBER IS " +
                    "rows_processed NUMBER;" +
                    "added_salary NUMBER;" +
                  "BEGIN " +
                    "added_salary := 0;" +
                    "rows_processed := 0;" +
                    "FOR emp IN (SELECT * FROM employee) LOOP " +
                      "rows_processed := rows_processed + 1;" +
                      "added_salary := added_salary + emp.salary / 100 * SALARY_PERCENT;" +
                    "END LOOP;" +
                    "UPDATE employee SET salary = salary + salary / 100 * SALARY_PERCENT;" +
                    "RETURN added_salary;" +
                  "END; " +
                "END pkg_employee; ";
    u.setSql(createPackageBody);
    u.execute();
    u.end();
  }  
}
