package com.your_domain.your_product.employee.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.math.BigDecimal;
import java.util.Date;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SpringLayout;

import com.jdbaccess.da.Transaction;
import com.jdbaccess.exception.ApplicationException;
import com.your_domain.your_product.address.transfer.Address;
import com.your_domain.your_product.employee.session.EmployeeSession;
import com.your_domain.your_product.employee.session.EmployeeSessionImpl;
import com.your_domain.your_product.employee.transfer.Employee;
import com.your_domain.your_product.employee.transfer.EmployeeVO;

/**
 * GUI-layer: main dialog for maintaining an employee value object
 */
public class EmployeeDialog extends JDialog {
  static final long serialVersionUID = EmployeeDialog.class.hashCode();
  private EmployeeVO empVO;
  private Transaction transaction;
  private JLabel nameLabel = new JLabel("Name:");
  private JTextField nameTextField = new JTextField();
  private JLabel salaryLabel = new JLabel("Salary:");
  private JTextField salaryTextField = new JTextField();
  private JLabel streetLabel = new JLabel("Street:");
  private JTextField streetTextField = new JTextField();
  private JLabel streetNoLabel = new JLabel("Street No.:");
  private JTextField streetNoTextField = new JTextField();
  private JLabel countryLabel = new JLabel("Country:");
  private JTextField countryTextField = new JTextField();
  private JLabel zipLabel = new JLabel("Zip-Code:");
  private JTextField zipTextField = new JTextField();
  private JLabel cityLabel = new JLabel("City:");
  private JTextField cityTextField = new JTextField();
  private JLabel descriptionLabel = new JLabel("Description:");
  private JTextArea descriptionTextArea = new JTextArea();
  private JLabel pictureLabel = new JLabel("Picture:");
  private JLabel pictureLabelImage = new JLabel("");
  private JButton cancelButton = new JButton("Cancel");
  private JButton okButton = new JButton("OK");
  private EmployeesFrame fatherFrame;
    
  public EmployeeDialog(EmployeesFrame fatherFrame, EmployeeVO empVO, Transaction t) {
    super(fatherFrame, "JDBaccess-Example: Employee", true);
    this.transaction = t;
    this.empVO = empVO;
    this.fatherFrame = fatherFrame;
    initData();
    initGui();
    initListeners(); 
    setVisible(true);
  }

  private void initGui() {
    JPanel panelNorth = createNorthPanel();
    JPanel panelCenter = createCenterPanel();
    JPanel panelSouth = createSouthPanel();
    getContentPane().add(panelNorth, BorderLayout.NORTH);
    getContentPane().add(panelCenter, BorderLayout.CENTER);
    getContentPane().add(panelSouth, BorderLayout.SOUTH);
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    pack();
    setLocation(fatherFrame.getLocation().x + 50, fatherFrame.getLocation().y - 50);
  }

  private JPanel createNorthPanel() {
    SpringLayout springLayout = new SpringLayout();
    JPanel panel_north = new JPanel(springLayout);
    panel_north.add(nameLabel);
    panel_north.add(nameTextField);
    panel_north.add(salaryLabel);
    panel_north.add(salaryTextField);
    panel_north.add(streetLabel);
    panel_north.add(streetTextField);
    panel_north.add(streetNoLabel);
    panel_north.add(streetNoTextField);
    panel_north.add(countryLabel);
    panel_north.add(countryTextField);
    panel_north.add(zipLabel);
    panel_north.add(zipTextField);
    panel_north.add(cityLabel);
    panel_north.add(cityTextField);
    SpringUtilities.makeCompactGrid(panel_north, 7, 2, 6, 6, 6, 6); //rows, cols, initX, initY, xPad, yPad
    return panel_north;
  }

  private JPanel createCenterPanel() {
    FlowLayout flowLayout = new FlowLayout(FlowLayout.LEFT);
    JPanel panel_center = new JPanel(flowLayout);
    descriptionTextArea.setRows(5);
    descriptionTextArea.setColumns(20);
    JScrollPane descScrollPane = 
      new JScrollPane(descriptionTextArea, 
                      JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, 
                      JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    panel_center.add(descriptionLabel);
    panel_center.add(descScrollPane);
    panel_center.add(pictureLabel);
    panel_center.add(pictureLabelImage);
    return panel_center;
  }
  
  private JPanel createSouthPanel() {
    FlowLayout flowLayout = new FlowLayout(FlowLayout.RIGHT);
    JPanel panel_south = new JPanel(flowLayout);
    panel_south.add(cancelButton);
    panel_south.add(okButton);
    return panel_south;
  }

  private void initData() {
    try {
      setData();
    } catch (ApplicationException e) {
      System.out.println(e.toString());
    }
  }

  private void setData() throws ApplicationException {
    Employee emp = empVO.getEmployee();
    nameTextField.setText(emp.getName());
    salaryTextField.setText(emp.getSalary().toString());
    Address firstAddress = (Address) empVO.getAddresses().get(0);
    streetTextField.setText(firstAddress.getStreet());
    streetNoTextField.setText(firstAddress.getStreetNumber());
    countryTextField.setText(firstAddress.getCountry());
    zipTextField.setText(firstAddress.getZipCode());
    cityTextField.setText(firstAddress.getCity());
    descriptionTextArea.setText(emp.getDescription());
    byte[] picture = emp.getPicture();
    Image pictureImage = Toolkit.getDefaultToolkit().createImage(picture);
    pictureLabelImage = new JLabel(new ImageIcon(pictureImage));
  }
  
  private void getData() throws ApplicationException {
    Date now = new Date();
    Employee emp = empVO.getEmployee();
    emp.setValueModified("name", nameTextField.getText());
    String salText = salaryTextField.getText();
    BigDecimal salary = new BigDecimal(salText);
    emp.setValueModified("salary", salary);
    emp.setValueModified("description", descriptionTextArea.getText());
    emp.setValueModified("modification", now);
    Address firstAddress = (Address) empVO.getAddresses().get(0);
    firstAddress.setValueModified("street", streetTextField.getText());
    firstAddress.setValueModified("streetNumber", streetNoTextField.getText());
    firstAddress.setValueModified("country", countryTextField.getText());
    firstAddress.setValueModified("zipCode", zipTextField.getText());
    firstAddress.setValueModified("city", cityTextField.getText());
    firstAddress.setValueModified("modification", now);
  }
  
  private void save() {
    try {
      getData();
      EmployeeSession empSession = new EmployeeSessionImpl(transaction);
      empSession.update(empVO);  
    } catch (ApplicationException e) {
      handle(e);
    }
  }
  
  private void initListeners() {
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        close();
      }
    });
    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        save();
        close();
        fatherFrame.refreshFromDB();
      }
    });
  }

  private void close() {
    dispose();
  }
  
  private void handle(ApplicationException e) {
    System.out.println(e.toString());
  }
  
}
