package com.your_domain.your_product.employee.session;

import java.util.ArrayList;

import com.jdbaccess.da.Result;
import com.jdbaccess.exception.ApplicationException;
import com.your_domain.your_product.employee.transfer.Employee;
import com.your_domain.your_product.employee.transfer.EmployeeVO;

/**
 * Session-layer.
 * Interface class of EmployeeSessionImpl. 
 * Business methods which change database contents in one transaction do 
 * a commit at the end of the method.
 */
public interface EmployeeSession {

  /**
   * Inserts a new employee value object (EmployeeVO with a new employee and new addresses).
   * Each address is inserted.
   * Finally does a commit.
   * @param empVO
   * @return the new inserted <EmployeeVO> object
   * @throws ApplicationException
   */
  public EmployeeVO insert(EmployeeVO empVO) throws ApplicationException;

  /**
   * Updates an employee value object (with an employee and addresses).
   * Each address is updated.
   * Finally does a commit.
   * @param empVO
   * @throws ApplicationException
   */
  public void update(EmployeeVO empVO) throws ApplicationException;

  /**
   * Finds an employee value object by an id
   * @param empId
   * @return the <EmployeeVO> object if it exists
   * @throws ApplicationException
   */
  public EmployeeVO findById(Long empId) throws ApplicationException;

  /**
   * Finds an employee value object by a name
   * @param empName
   * @return the <EmployeeVO> object if it exists
   * @throws ApplicationException
   */
  public EmployeeVO findByName(String empName) throws ApplicationException;

  /**
   * Finds all employees as a <Result> object. 
   * @param orderBy <ArrayList> of field names 
   * @return <Result> of the query.
   * @throws ApplicationException
   */
  public Result findEmployeeAll(ArrayList orderBy) throws ApplicationException;

  /**
   * Finds an employees by id. 
   * @param id id of the employee 
   * @return found employee
   * @throws ApplicationException
   */
  public Employee findEmployeeById(Long id) throws ApplicationException;

  /**
   * Finds an employees by name. 
   * @param name name of the employee 
   * @return found employee
   * @throws ApplicationException
   */
  public Result findEmployeeByName(String name) throws ApplicationException;

  /**
   * Finds all employees which have a salary less than the specified salary. 
   * @param salaray salary of the employee 
   * @return found employee
   * @throws ApplicationException
   */
  public Result findEmployeeBySalaryLessThan(Number salary)
      throws ApplicationException;

}