package com.your_domain.your_product.main;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;

import com.jdbaccess.da.CallResult;
import com.jdbaccess.da.DAFactory;
import com.jdbaccess.da.Delete;
import com.jdbaccess.da.Function;
import com.jdbaccess.da.Insert;
import com.jdbaccess.da.Procedure;
import com.jdbaccess.da.Select;
import com.jdbaccess.da.Result;
import com.jdbaccess.da.Transaction;
import com.jdbaccess.da.Update;
import com.jdbaccess.db.JDBaccess;
import com.jdbaccess.exception.ApplicationException;
import com.jdbaccess.sql.ComparisonCondition;
import com.jdbaccess.sql.LogicalCondition;
import com.jdbaccess.transfer.Field;
import com.jdbaccess.transfer.Row;
import com.jdbaccess.transfer.TransferObject;
import com.your_domain.your_product.address.dao.AddressDao;
import com.your_domain.your_product.address.dao.AddressDaoImpl;
import com.your_domain.your_product.address.transfer.Address;
import com.your_domain.your_product.employee.dao.EmpAddrDao;
import com.your_domain.your_product.employee.dao.EmpAddrDaoImpl;
import com.your_domain.your_product.employee.dao.EmployeeDao;
import com.your_domain.your_product.employee.dao.EmployeeDaoImpl;
import com.your_domain.your_product.employee.session.EmployeeSession;
import com.your_domain.your_product.employee.session.EmployeeSessionImpl;
import com.your_domain.your_product.employee.transfer.Employee;
import com.your_domain.your_product.employee.transfer.EmployeeVO;

/**
 * Example for using data access operations (insert, update, delete, select, function 
 * and procedure). 
 */
public class ExampleDA {
  public static void main (String args []) {
    try {
      ExampleJDBAccess jdbAccess = new ExampleJDBAccess();
      jdbAccess.start();
      
      ExampleDA exampleDA = new ExampleDA();
      exampleDA.createOrReplace();
      exampleDA.insert();
      exampleDA.update();
      exampleDA.select();
      exampleDA.delete();
      exampleDA.executePLSQL();

      jdbAccess.end();
    } catch (ApplicationException e) {
      System.out.println(e.toString());
    }
  }
  
  public void createOrReplace() {
    try {
      Transaction t = DAFactory.getTransaction();
      t.begin();
      EmployeeDao empDao = new EmployeeDaoImpl(t);
      AddressDao addrDao = new AddressDaoImpl(t);
      EmpAddrDao empAddrDao = new EmpAddrDaoImpl(t);
      try {empDao.dropTable();} catch (ApplicationException e) {} // if it does not exist do nothing
      try {empDao.dropSequence();} catch (ApplicationException e) {} // if it does not exist do nothing
      try {addrDao.dropTable();} catch (ApplicationException e) {} // if it does not exist do nothing
      try {addrDao.dropSequence();} catch (ApplicationException e) {} // if it does not exist do nothing
      try {empAddrDao.dropTable();} catch (ApplicationException e) {} // if it does not exist do nothing
      empDao.createTable();
      empDao.createSequence();
      empDao.createPackage();
      addrDao.createTable();
      addrDao.createSequence();
      empAddrDao.createTable();
      t.commit();
      t.end();
    } catch (ApplicationException e) {
      e.printStackTrace();
    }
  }
  
  public void insert() {
    try {
      Transaction t = DAFactory.getTransaction();
      t.begin();
      
      // create an employee (own sql string)
      String sqlInsertString = "insert into employee(id,name,salary,creation,modification) values(?,?,?,?,?)";
      Insert i1 = DAFactory.getInsert(t);
      i1.setSql(sqlInsertString);
      Long id1 = getEmployeeSequenceNextVal(t);
      ArrayList values1 = new ArrayList();
      values1.add(id1);
      values1.add("berta");
      values1.add(new Long(14000));
      Date now = new Date();
      values1.add(now);
      values1.add(now);
      i1.setParameters(values1);
      i1.execute();
      Long size1 = i1.getSize();
      i1.end();
      
      // same insert but read by object type and name in an XML file
      Insert i2 = DAFactory.getInsert(t, "employee", "insertWithoutLobs", EmployeeDaoImpl.class);
      Long id2 = getEmployeeSequenceNextVal(t);
      ArrayList values2 = new ArrayList();
      values2.add(id2);
      values2.add("joe");
      values2.add(new Long(19000));
      values2.add(now);
      values2.add(now);
      i2.setParameters(values2);
      i2.execute();
      i2.end();

      // insert an employee (description is clob and picture is blob) 
      Insert i3 = DAFactory.getInsert(t);
      Long id3 = getEmployeeSequenceNextVal(t);
      Employee emp3 = new Employee();
      emp3.setValueModified("id", id3);
      emp3.setValueModified("name", "john");
      emp3.setValueModified("salary", new BigDecimal(24000));
      String description = getExampleString(); // example string
      emp3.setValueModified("description", description);
      byte[] picture = getExampleBytes(); // example logo bytes
      emp3.setValueModified("picture", picture);
      emp3.setValueModified("creation", now);
      emp3.setValueModified("modification", now);
      i3.setTO(emp3);
      Object returnObject3 = i3.execute();
      i3.end();
      
      EmployeeDao empDao = new EmployeeDaoImpl(t);

      // insert an employee with EmployeeDao; id is generated automatically by sequence
      TransferObject returnObject = empDao.insert("mary", new BigDecimal(29000), "description", picture, now, now);

      // insert two employees with EmployeeDao (two row objects batched)
      ArrayList rows = new ArrayList();
      Row row1 = new Row();
      row1.setValueModified("name", "peter");
      row1.setValueModified("salary", new Long(34000));
      row1.setValueModified("description", "description .....");
      row1.setValueModified("picture", picture);
      row1.setValueModified("creation", now);
      row1.setValueModified("modification", now);
      Row row2 = new Row();
      row2.setValueModified("name", "ted");
      row2.setValueModified("salary", new Long(40000));
      row2.setValueModified("description", "description .....");
      row2.setValueModified("picture", picture);
      row2.setValueModified("creation", now);
      row2.setValueModified("modification", now);
      rows.add(row1);
      rows.add(row2);
      ArrayList returnRows1 = empDao.insert(rows);

      // insert two employees with EmployeeDao (two employees batched) 
      ArrayList emps = new ArrayList();
      Employee emp1 = new Employee();
      emp1.setValueModified("name", "tim");
      BigDecimal timSal = BigDecimal.valueOf(5000002, 2); // 50000.02
      emp1.setValueModified("salary", timSal);
      emp1.setValueModified("description", "description .....");
      emp1.setValueModified("picture", picture);
      emp1.setValueModified("creation", now);
      emp1.setValueModified("modification", now);
      Employee emp2 = new Employee();
      emp2.setValueModified("name", "tina");
      BigDecimal tinaSal = BigDecimal.valueOf(5500002, 2); // 55000.02
      emp2.setValueModified("salary", tinaSal);
      emp2.setValueModified("description", "description .....");
      emp2.setValueModified("picture", picture);
      emp2.setValueModified("creation", now);
      emp2.setValueModified("modification", now);
      emps.add(emp1);
      emps.add(emp2);
      ArrayList returnObject2 = empDao.insert(emps);

      // insert an employee and its address with EmployeeSession (one EmployeeVO)
      EmployeeSession empSession = new EmployeeSessionImpl(t);
      EmployeeVO empVO = new EmployeeVO();
      Employee emp = new Employee();
      emp.setValueModified("name", "tommy");
      emp.setValueModified("salary", new BigDecimal(100000));
      emp.setValueModified("description", "description"); // clob value
      emp.setValueModified("picture", picture);
      emp.setValueModified("creation", now);
      emp.setValueModified("modification", now);
      empVO.setEmployee(emp);
      Address addr = new Address();
      addr.setValueModified("street", "Street A");
      addr.setValueModified("streetNumber", "A");
      addr.setValueModified("country", "US");
      addr.setValueModified("zipCode", "10001");
      addr.setValueModified("city", "New York");
      addr.setValueModified("creation", now);
      addr.setValueModified("modification", now);
      empVO.addAddress(addr);
      EmployeeVO newEmpVO = empSession.insert(empVO);

      t.commit();
      t.end();
    } catch (ApplicationException e) {
      e.printStackTrace();
    }
  }

  public void update() {
    try {
      Transaction t = DAFactory.getTransaction();
      t.begin();

      // update salary of an employee (own sql) by id
      Update u1 = DAFactory.getUpdate(t);
      String sqlStringUpdateEmp = "update employee set salary = ? where id = ?";
      u1.setSql(sqlStringUpdateEmp);
      ArrayList values1 = new ArrayList();
      values1.add(new BigDecimal(14500));
      values1.add(new Long(1));
      u1.setParameters(values1);
      u1.execute();
      u1.end();
      
      // update salary of an employee by id 
      Update u2 = DAFactory.getUpdate(t);
      Employee emp1 = new Employee();
      emp1.setId(new Long(2));
      emp1.setValueModified("salary", new BigDecimal(20000));
      u2.setTO(emp1);
      u2.execute();
      u2.end();

      // update salary of an employee by name 
      Update u3 = DAFactory.getUpdate(t);
      Employee empMary = new Employee();
      empMary.setName("mary");
      empMary.setValueModified("salary", new BigDecimal(30000));
      ComparisonCondition nameEqualsComparison = new ComparisonCondition("name", ComparisonCondition.EQUALS, null);
      LogicalCondition nameEqualsCondition = new LogicalCondition(nameEqualsComparison, LogicalCondition.EMPTY, null);
      empMary.setWhereCondition(nameEqualsCondition);
      u3.setTO(empMary);
      u3.execute();
      u3.end();

      // update salary of an employee by id (Update is read by DAFactory by object type and name in XML files)
      Update u4 = DAFactory.getUpdate(t, "employee", "updateSalaryById", EmployeeDaoImpl.class);
      ArrayList values4 = new ArrayList();
      values4.add(new BigDecimal(25000));
      values4.add(new Long(3));
      u4.setParameters(values4);
      u4.execute();
      u4.end();

      
      EmployeeDao empDao = new EmployeeDaoImpl(t);

      // update salary of an employee (row object) by id with EmployeeDao 
      Row row1 = new Row();
      row1.setEntity("employee");
      row1.setValue("id", new Long(1));
      row1.setValueModified("name", "berta\u2122"); // unicode char for trademark
      row1.setValueModified("salary", new BigDecimal(15000));
      empDao.update(row1);
      
      // update of an employee (row object) by name with EmployeeDao
      Row row2 = new Row();
      row2.setEntity("employee");
      row2.setValue("name", "tina");
      row2.setValueModified("salary", new BigDecimal(55000));
      byte[] picture = getExampleBytes();
      row2.setValueModified("description", "updated description ....");
      row2.setValueModified("picture", picture);
      row2.setWhereCondition(nameEqualsCondition);
      empDao.update(row2);
      
      // update of an employee by name with EmployeeDao
      empDao.updateSalaryByName("tim", new BigDecimal(50000));

      // update of two employees by id with EmployeeDao
      Employee emp2 = new Employee();
      emp2.setId(new Long(2));
      emp2.setValueModified("salary", new BigDecimal(20000));
      Employee emp3 = new Employee();
      emp3.setId(new Long(3));
      emp3.setValueModified("salary", new BigDecimal(25000));
      empDao.update(emp2);
      empDao.update(emp3);
      
      // update of an amployee by name with EmployeeDao
      Employee empPeter = new Employee();
      empPeter.setName("peter");
      empPeter.setValueModified("salary", new BigDecimal(35000));
      empPeter.setWhereCondition(nameEqualsCondition);
      empDao.update(empPeter);

      t.commit();

      // new second Transaction for an update with EmployeeDao
      Transaction t2 = DAFactory.getTransaction();
      t2.begin();
      EmployeeDao empDao2 = new EmployeeDaoImpl(t2);
      empDao2.updateSalaryById(new Long(3), BigDecimal.valueOf(1500005, 2));
      t2.commit();
      t2.end();
      
      // update an employee (EmployeeVO) with EmployeeSession
      EmployeeSession empSession = new EmployeeSessionImpl(t);
      EmployeeVO tomVO = empSession.findByName("tommy");
      if (tomVO != null) {
        Employee tom = tomVO.getEmployee();
        tom.setValueModified("name", "tom");
        empSession.update(tomVO);
      }

      t.end();

    } catch (ApplicationException e) {
      e.printStackTrace();
    }
  }

  public void select() {
    try {
      // find employees by salary (own sql)
      String sqlStringEmployeeBySalary = "select * from employee where salary < ?";
      Select s1 = DAFactory.getSelect();
      s1.setObjectType("employee");
      s1.setName("bySalary");
      s1.setSql(sqlStringEmployeeBySalary);
      s1.setResultType(Employee.class);
      ArrayList params1 = new ArrayList();
      params1.add(new Long(100000));
      s1.setParameters(params1);
      Result r1 = s1.execute();
      Long size1 = r1.getSize();
      Employee firstElement1 = (Employee)r1.getFirstElement();
      if (firstElement1 != null) {
        BigDecimal salary1 = (BigDecimal) firstElement1.getValue("salary");
      }
      ArrayList l1 = r1.getAllElements();
      r1.setPosition(0); // position before first element again
      l1 = r1.getNextElements(size1.longValue()); // get elements starting at position 1 again
      long pos = r1.getPosition(); // position is on last element of result
      s1.end();
      
      // same select again
      Select s2 = DAFactory.getSelect();
      s2.setSql(sqlStringEmployeeBySalary);
      s2.setResultType(Employee.class);
      s2.setParameters(params1);
      Result r2 = s2.execute();
      ArrayList l2 = r2.getAllElements();
      s2.end();
      
      // find average salary of all employees
      Select s3 = DAFactory.getSelect();
      s3.setObjectType("employee");
      s3.setName("averageSalary");
      String sqlStringAverageSalary = "select sum(salary) / count(id) as average_salary from employee";
      s3.setSql(sqlStringAverageSalary);
      Result r3 = s3.execute();
      TransferObject averageSalary = r3.getFirstElement();
      s3.end();
      
      // find all employees with Select (own sql)
      String sqlStringEmployeeAll = "select * from employee";
      Select s4 = DAFactory.getSelect();
      s4.setObjectType("employee");
      s4.setName("all");
      s4.setSql(sqlStringEmployeeAll);
      // order by "salary desc" and "name"
      ArrayList orderBy = new ArrayList();
      orderBy.add("salary desc");
      orderBy.add("name");
      s4.setOrderBy(orderBy);
      Result r4 = s4.execute();
      // Clobs and Blobs of that result are read full (not only first chunks) in afterwards getNextElement operations
      r4.setReadLobsFull(true);
      ArrayList l4 = r4.getAllElements();
      s4.end();

      // same select again but without order by
      Select s5 = DAFactory.getSelect();
      s5.setObjectType("employee");
      s5.setName("all");
      s5.setSql(sqlStringEmployeeAll);
      Result r5 = s5.execute();
      ArrayList l5 = r5.getAllElements();  
      s5.end();
      
      // same select again
      Select s6 = DAFactory.getSelect();
      s6.setObjectType("employee");
      s6.setName("all");
      s6.setSql(sqlStringEmployeeAll);
      Result r6 = s6.execute();
      ArrayList l6 = r6.getAllElements();
      Long size6 = r6.getSize();
      s6.end();
      
      // same select 3 times
      Select s7 = DAFactory.getSelect("employee", "all", EmployeeDao.class);
      Result r7 = s7.execute();
      Long size7 = r7.getSize();
      ArrayList l7 = r7.getAllElements();  
      s7.end();
      Result r8 = s7.execute();
      Long size8 = r8.getSize();
      ArrayList l8 = r8.getAllElements();  
      s7.end();
      Result r9 = s7.execute();
      Long size9 = r9.getSize();
      ArrayList l9 = r9.getAllElements(); 
      s7.end();

      
      // EmployeeDao without transaction
      EmployeeDao empDao = new EmployeeDaoImpl();
      
      // 3 times same findEmployeeBySalary with EmployeeDao
      Long salary = new Long(100000);
      Result r10 = empDao.findBySalaryLessThan(salary);      
      Long size10 = r10.getSize();
      TransferObject firstElement10 = r10.getFirstElement();
      ArrayList l10a = r10.getAllElements();
      ArrayList l10b = r10.getAllElements(); 
      r10.close(); // end of result and end of select
      Result r11 = empDao.findBySalaryLessThan(salary);        
      ArrayList l11 = r11.getAllElements(); 
      r11.close();
      Result r12 = empDao.findBySalaryLessThan(salary);       
      ArrayList l12 = r12.getAllElements(); 
      r12.close();
      
      // 4 same finds with EmployeeDao: select is searched by type and name in da.xml; results are cached
      Result r13 = empDao.find("employee", "all");
      Long size13 = r13.getSize();
      ArrayList l13 = r13.getAllElements();
      r13.close();
      Result r14 = empDao.find("employee", "all");
      Long size14 = r14.getSize();
      ArrayList l14 = r14.getAllElements(); 
      r14.close();
      Result r15 = empDao.find("employee", "all");
      Long size15 = r15.getSize();
      ArrayList l15 = r15.getAllElements(); 
      r15.close();
      Result r16 = empDao.find("employee", "all");
      Long size16 = r16.getSize();
      ArrayList l16 = r16.getAllElements();
      r16.close();

      // 2 finds with type, name, sql-select
      Result r17 = empDao.find("employee", "all", "select id, name from employee", null, null, null, null, null, false);
      ArrayList l17 = r17.getAllElements();  // get next elements as page size is defined
      r17.close();
      Result r18 = empDao.find("employee", "all", "select * from employee", null, null, null, null, null, false);
      ArrayList l18 = r18.getAllElements();  // get next elements as page size is defined
      r18.close();
      
      // find with type, name, sql-select, orderBy, resultFieldTypes
      ArrayList orderBy19 = new ArrayList();
      orderBy19.add("name");
      ArrayList resultFieldTypes19 = new ArrayList();
      Field id = new Field("id", Integer.class);  // specified result type Integer for id
      resultFieldTypes19.add(id);
      Field sal = new Field("salary", Double.class);  // specified result type Double for salary
      resultFieldTypes19.add(sal);
      Result r19 = empDao.find("employee", "all", "select * from employee", null, null, orderBy19, null, resultFieldTypes19, true);
      Row rowDao19 = (Row) r19.getFirstElement();
      Double salary19_1 = (Double) rowDao19.getValue("salary");
      Double salary19_2 = (Double) rowDao19.getValue("SALARY"); // getValue is case insensitive
      r19.close();
      
      // select for emp-addresses read in an XML file
      Select s20 = DAFactory.getSelect("emp-address", "all", EmployeeDao.class);
      Result r20 = s20.execute();
      ArrayList l20 = r20.getAllElements();
      s20.end();

      // EmployeeDB with a new transaction for updating data
      Transaction t = DAFactory.getTransaction();
      t.begin();
      EmployeeDao empDaoWithTransaction = new EmployeeDaoImpl(t);
      Date now = new Date();
      // create a test employee which is rolled back after following selects
      empDaoWithTransaction.insert("test", new BigDecimal(200), null, null, now, now);
      // findBySalary of this temporarily inserted employee in that transaction
      Long smallSalary = new Long(1000);
      Result r21 = empDaoWithTransaction.findBySalaryLessThan(smallSalary);
      ArrayList l21 = r21.getAllElements();
      r21.close();
      // findBySalary of this temporarily inserted employee in another transaction
      Result r22 = empDao.findBySalaryLessThan(smallSalary);
      boolean isEmpty = r22.isEmpty(); // is empty because 200 is inserted in another transaction which has not been committed
      ArrayList l22 = r22.getAllElements();  
      r22.close();

      t.rollback();
      t.end();

    } catch (ApplicationException e) {
      e.printStackTrace();
    }
  }

  public void delete() {
    try {
      Transaction t1 = DAFactory.getTransaction();
      t1.begin();

      // delete employees by name (with whereCondition) 
      Delete d1 = DAFactory.getDelete(t1);
      Employee empMary = new Employee();
      empMary.setName("mary");
      ComparisonCondition nameEqualsComparison = new ComparisonCondition("name", ComparisonCondition.EQUALS, null);
      LogicalCondition nameEqualsCondition = new LogicalCondition(nameEqualsComparison, LogicalCondition.EMPTY, null);
      empMary.setWhereCondition(nameEqualsCondition);
      d1.setTO(empMary);
      d1.execute();
      d1.end();

      // find an employee by id (own sql)
      ArrayList values = new ArrayList();
      values.add(new Long(1));
      Select s = DAFactory.getSelect();
      s.setObjectType("employee");
      s.setName("byId");
      s.setSql("select * from employee where id = ?");
      s.setResultType(Employee.class);
      s.setParameters(values);
      Result r = s.execute();
      Employee empSelected = (Employee) r.getFirstElement();
      s.end();

      // delete an employee by id (Employee; where condition is automatically id) 
      if (empSelected != null) {
        Delete d2 = DAFactory.getDelete(t1);
        d2.setTO(empSelected);
        d2.execute();
        d2.end();
      }
      
      EmployeeDao empDao1 = new EmployeeDaoImpl(t1);

      // delete employee by name with empDB
      empDao1.delete(empMary);

      // still finds "mary" in another transaction. But "mary" is db-locked. You could update "mary" 
      // only in an new transaction t2 if the first transaction t1 has a commit. 
      // But deletion of "peter" is possible in a new transaction.
      EmployeeDao empDaoWithoutTransaction = new EmployeeDaoImpl();
      Result r2 = empDaoWithoutTransaction.findByName("mary");
      Employee empMary2 = (Employee) r2.getFirstElement(); 
      r2.close();        
      Result r3 = empDaoWithoutTransaction.findByName("peter");
      Employee empPeter = (Employee) r3.getFirstElement(); 
      r3.close();
      Transaction t2 = DAFactory.getTransaction();
      t2.begin();
      EmployeeDao empDao2 = new EmployeeDaoImpl(t2);
      empDao2.delete(empPeter);
      t2.commit();
      t2.end();
      
      // delete an employee (with Employee and whereCondition) 
      Delete d3 = DAFactory.getDelete(t1);
      Employee emp = new Employee();
      emp.setName("j%");
      emp.setSalary(new BigDecimal(15000));
      ComparisonCondition nameLikeComparison = new ComparisonCondition("name", ComparisonCondition.LIKE, null);
      ComparisonCondition salLessComparison = new ComparisonCondition("salary", ComparisonCondition.LESSTHAN, null);
      LogicalCondition nameLikeOrSalaryLessCondition = new LogicalCondition(nameLikeComparison, LogicalCondition.OR, salLessComparison);
      emp.setWhereCondition(nameLikeOrSalaryLessCondition);
      d3.setTO(emp);
      d3.execute();
      d3.end();

      t1.commit();

      // delete all employees (own sql)
      Delete d4 = DAFactory.getDelete(t1);
      d4.setSql("delete from employee");
      d4.execute();
      d4.end();
      t1.rollback();

      // delete employees by name (own sql)
      Delete d5 = DAFactory.getDelete(t1);
      d5.setSql("delete from employee where name = ?");
      ArrayList valuesName = new ArrayList();
      valuesName.add("bla");
      d5.setParameters(valuesName);
      d5.execute();
      d5.end();
      t1.rollback();

      t1.end();

    } catch (ApplicationException e) {
      e.printStackTrace();
    }
  }

  public void executePLSQL() {
    try {
      Transaction t = DAFactory.getTransaction();
      t.begin();
      EmployeeDao empDao = new EmployeeDaoImpl(t);

      // Oracle PL/SQL-Procedure call with EmployeeDao: add 5% to salary of all employees
      ArrayList params1 = new ArrayList();
      params1.add(new Integer(5));
      ArrayList outputParamTypes1 = new ArrayList();
      outputParamTypes1.add("varchar");
      outputParamTypes1.add("numeric");
      outputParamTypes1.add("cursor");
      outputParamTypes1.add("cursor"); 
      Procedure p1 = empDao.executeProcedure("employee", "prc_addSalaryPercent", "pkg_employee", "prc_addSalaryPercent", params1, outputParamTypes1);
      ArrayList outputParams1 = p1.getOutputParams(); 
      Object success1 = outputParams1.get(0);
      Object addedSalary = outputParams1.get(1);
      CallResult r11 = p1.getResult(1);  // first result: employees before execution of procedure
      ArrayList elems11 = r11.getAllElements();
      CallResult r12 = p1.getResult(2); // second result: employees after execution of procedure
      ArrayList elems12 = r12.getAllElements();
      Long updateCount = p1.getSize(); // count of all updated rows of this procedure
      p1.end();
      t.rollback();
      
      // same Oracle PL/SQL-Procedure call without EmployeeDao
      Procedure p2 = DAFactory.getProcedure(t);
      p2.setObjectType("employee");
      p2.setModuleName("pkg_employee");
      p2.setProcedureName("prc_addSalaryPercent");
      p2.setParameters(params1);
      p2.setOutputParamTypes(outputParamTypes1);
      ArrayList outputParams2 = (ArrayList) p2.execute(); // result of the execution is the array of output parameters of the procedure
      Object success2 = outputParams2.get(0);
      CallResult r21 = p2.getResult(1);
      ArrayList elems21 = r21.getAllElements();
      CallResult r22 = p2.getResult(2);
      ArrayList elems22 = r22.getAllElements();
      p2.end();
      t.rollback();
      
      // same Oracle PL/SQL-Procedure call read in the XML file
      Procedure p3 = DAFactory.getProcedure(t, "employee", "prc_addSalaryPercent", EmployeeDao.class);
      p3.setParameters(params1);
      ArrayList outputParams3 = (ArrayList) p3.execute();  // result of the execution is the array of output parameters of the procedure
      Object success3 = outputParams3.get(0);
      p3.end();
      t.rollback();

      // dynamic PL/SQL-Procedure: insert an employee and returns the new generated sequence id of that insert
      Procedure p4 = DAFactory.getProcedure(t, "employee", "prc_insert", EmployeeDao.class);
      ArrayList insertParams = new ArrayList();
      insertParams.add("zorro");
      insertParams.add(new BigDecimal(10000000));
      insertParams.add(new Date());
      insertParams.add(new Date());
      p4.setParameters(insertParams);
      Object seqId = p4.execute();  // result of the execution is the first element of the array of output parameters of the procedure
      p4.end();
      t.rollback();
      
      // dynamic PL/SQL-Procedure: insert an employee and returns the modification date of that insert
      Procedure p5 = DAFactory.getProcedure(t, "employee", "prc_insertRetLastMod", EmployeeDao.class);
      ArrayList insertParamsp5 = new ArrayList();
      insertParamsp5.add("zorro2");
      insertParamsp5.add(new BigDecimal(10000000));
      insertParamsp5.add(new Date());
      insertParamsp5.add(new Date());
      p5.setParameters(insertParamsp5);
      Object modificationDate = p5.execute();  // result of the execution is the first element of the array of output parameters of the procedure
      p5.end();
      t.rollback();
      
      // Oracle PL/SQL-Function call with EmployeeDao: add 5% to salary of all employees and return the sum of all added salary
      String outputParamType = "numeric";
      Function f1 = empDao.executeFunction("employee", "fnc_addSalaryPercent", "pkg_employee", "fnc_addSalaryPercent", params1, outputParamType);
      Object addedSalary1 = f1.getFunctionResult();
      f1.end();
      t.rollback();

      // same Oracle PL/SQL-Function call without EmployeeDao
      Function f2 = DAFactory.getFunction(t);
      f2.setObjectType("employee");
      f2.setModuleName("pkg_employee");
      f2.setFunctionName("fnc_addSalaryPercent");
      f2.setParameters(params1);
      f2.setOutputParamType(outputParamType); // sql type of the result of the function execution 
      Object addedSalary2 = f2.execute();
      f2.end();
      t.rollback();

      // same Oracle PL/SQL-Function call read in the XML file
      Function f3 = DAFactory.getFunction(t, "employee", "fnc_addSalaryPercent", EmployeeDao.class);
      f3.setParameters(params1);
      Object addedSalary3 = f3.execute();
      f3.end();

      t.commit();
      t.end();

    } catch (ApplicationException e) {
      e.printStackTrace();
    }
  }
  
  private Long getEmployeeSequenceNextVal(Transaction t) throws ApplicationException {
    String seqName = "seq_employee";
    Long nextVal = null;
    Select s = DAFactory.getSelect(t);
    s.setSql("select " + seqName + ".nextval from dual");
    Result r = s.execute();
    TransferObject dao = (TransferObject) r.getFirstElement();
    s.end();
    if (dao != null) {
      BigDecimal nextValBigDec = (BigDecimal) dao.getValue("nextval");
      nextVal = new Long(nextValBigDec.longValue());
    }
    return nextVal;
  }
  
  private byte[] getExampleBytes() throws ApplicationException {
    return JDBaccess.getIconBytes();
  }

  private String getExampleString() {
    int size = 4000;
    StringBuffer buffer = new StringBuffer("");
    for (int i=0; i< size; i++)
      buffer.append("desc+desc+");
    String str = buffer.substring(0);
    return str;
  }  
  
}
